import java.awt.*;
import java.util.*;

// This subclass of Panel limits the Components available
// to be added to those that implement the
// QueryableComponent interface

class DBPanel extends Panel {

Database db;
Vector qryComps;

DBPanel(Database rdbms) {
    super();
    db = rdbms;
    qryComps = new Vector();
    setBackground(Color.lightGray);
    setLayout(new FlowLayout());
}

public void add(String placeHolder, String query,
                QueryableComponent comp, String label) {

	int i;
	String sizeStr;
	CompElement compe;
	Panel wPanel;
	Label wLabel;

    comp.setSQLText(db, query);
    comp.setDBPanel(this);

    compe = new CompElement(comp);
    compe.symbSQL = query;
    compe.placeHolder = placeHolder;

    qryComps.addElement(compe);
    calcDefining();
    substSQL();
    comp.refreshSQL(db, compe.comp.getSQLText());

    wPanel = new Panel();
    wPanel.setLayout(new BorderLayout());
    wLabel = new Label(label);
    wLabel.setAlignment(Label.CENTER);
    wPanel.add("North", wLabel);
	wPanel.add("South",(Component)comp);
	add(wPanel);
	validate();
}


public void calcDefining() {

        CompElement compe;
        String curTag;

        for (Enumeration e = qryComps.elements() ; e.hasMoreElements() ;) {
            compe = (CompElement)e.nextElement();
            Enumeration d = compe.dependentOn.keys();
            while ((compe.dependentOn.size() > 0) && (d.hasMoreElements()==true)) {
                curTag = (String)(d.nextElement());
                getCompElement(curTag).defines.put(compe.placeHolder, compe);
            }
        }
        for (Enumeration e = qryComps.elements() ; e.hasMoreElements() ;) {
            compe = (CompElement)e.nextElement();
        }
}


public CompElement getCompElement(String tag) {

        CompElement compe;
        String tag2;

        for (Enumeration e = qryComps.elements() ; e.hasMoreElements() ;) {
            compe = (CompElement)e.nextElement();
            tag2 = compe.placeHolder;
            if (tag.equals(compe.placeHolder)) {
                    return (CompElement)compe;
            }
        }
        return null;
}

public void substSQL() {

        CompElement compe;
        String curTag;
        int i=0;
        int j=0;
        int c=0;
        String sqlText;
        StringBuffer parsedSQL;
        String tag;

        for (Enumeration e = qryComps.elements() ; e.hasMoreElements() ;) {
            compe = (CompElement)e.nextElement();
            sqlText = compe.symbSQL;
            parsedSQL = new StringBuffer();
            while (i >= 0) {
                i = sqlText.indexOf(":",i);
                if (i >= 0) {
                    parsedSQL.append(sqlText.substring(c,i));
                    j = sqlText.indexOf(" ",i);
                    if (j < 0) {
                        j = sqlText.length()-1;
                    }
                    tag = new String(sqlText.substring(i+1,j));

                    parsedSQL.append(" ").append(getCompElement(tag).comp.getValue()).append(" ");
                    i = j;
                    c = j;
                }
            }
            parsedSQL.append(sqlText.substring(c));
            compe.comp.setSQLText(db, new String(parsedSQL));
            i=0;
            c=0;
        }
}


public boolean handleEvent(Event evt)
{
    QueryableComponent comp;
    CompElement compe;
    String curTag;

    switch(evt.id) {
        case Event.ACTION_EVENT: {
            if (evt.target.getClass().getInterfaces()[0].getName().equals("QueryableComponent")) {

                comp = (QueryableComponent)evt.target;
                for (Enumeration e = qryComps.elements() ; e.hasMoreElements() ;) {
                    substSQL();
                    compe = (CompElement)e.nextElement();
                    if (compe.comp == evt.target) {
                        refreshRecurs(compe, false);
                        validate();
                    }
                }
                return true;
               }
        }
        default:
            return super.handleEvent(evt);
    }
}

void refreshRecurs(CompElement compe , boolean refreshSelf) {

    String curTag;

    substSQL();
    // Refresh widget with passed in tag
    if (refreshSelf == true) {
        compe.comp.refreshSQL();
    }

    // Recursively refresh subordinate widgets
    Enumeration d = compe.defines.keys();
    while ((compe.defines.size() > 0) && (d.hasMoreElements()==true)) {
        curTag = (String)(d.nextElement());
        refreshRecurs(getCompElement(curTag), true);
    }
}

void handleChange() {

    CompElement compe;
    String curTag;

    calcDefining();
    substSQL();

    for (Enumeration e = qryComps.elements() ; e.hasMoreElements() ;) {
        compe = (CompElement)e.nextElement();
        compe.requery();
    }

	validate();

}

}
