import java.util.*;
import java.io.*;
import java.net.*;

public class Database {
  public static int MSG_OK = 0;
  public static int MSG_ERR = 1;
  public static int MSG_CLOSE = 2;
  public static int MSG_EXEC = 3;
  public static int MSG_CURSOR = 4;
  public static int MSG_QUERY = 5;

  String  host;
  int     port;
  String  dbname;
  String  user;
  String  passwd;
  public boolean open;
  public Socket  s;

  public DataInputStream din;
  public DataOutputStream dout;

  static int atoi(String s) {
    return (new Integer(s)).intValue();
  }

  public Database(String hostm, int portm, String db, String name, String pass) {
    dbname = db;
    user = name;
    passwd = pass;
    host = hostm;
    port = portm;
    connect();
  }

  public Database() {
    dbname = null;
    user = null;
    passwd = null;
    host = null;
    port = 0;
  }

  public void connect() {
	if (host == null) { return; }

	try {
		s = new Socket(host,port);
	} catch (IOException e) {
     		e.printStackTrace();
		System.err.println(e.getMessage());
   	}

	try {
		din = new DataInputStream(s.getInputStream());
	} catch (IOException e) {
      		e.printStackTrace();
	      	System.err.println(e.getMessage());
   	}

	try {
		dout = new DataOutputStream(s.getOutputStream());
	} catch (IOException e) {
      		e.printStackTrace();
	      	System.err.println(e.getMessage());
   	}

	System.out.println("Connected to "+host+" on port "+port);
  }

  public boolean close() {
    byte stat;

	if (host == null) { return true;}

	try {
	    dout.writeByte(MSG_CLOSE);
	    dout.writeByte('\n');
	    dout.flush();

    	    if ((stat = din.readByte()) != MSG_OK) {
        	System.err.println("Didn't get ACK on Close ("+stat+")");
    	    return(false);
	    }
   	} catch (IOException e) {
    	e.printStackTrace();
      	System.err.println(e.getMessage());
   	}

    return(true);
  }

  public boolean exec(String sql) {
    byte stat;

	if (host == null) { return true;}

	try {
	    dout.writeByte(MSG_EXEC);
    	dout.writeUTF(sql);
	    dout.flush();
        if ((stat = din.readByte()) != MSG_OK) {
			System.err.println("Didn't get ACK on EXEC >"+stat+"<XS");
        	din.readByte();
        	return(false);
    	}
    	din.readByte();
    } catch (IOException e) {
    	e.printStackTrace();
      	System.err.println(e.getMessage());
    }
    return(true);
  }
}
