/*  No copyright is claimed [For this document], but the author requests the following
	information be included anywhere this code is used.
	written 1996 as part of submission for Sun's Java Cup...questions & comments happily accepted.
	Miles T. Parker, Software Development & Consulting
	Washington, DC. miles@clark.net (202) 232-1468												*/	
	
import java.awt.*;
import java.util.*;
import java.io.*;

class CatoCanvas extends Canvas {

	public DrawableCatoTables catoTables;
	
	CatoCanvas (DrawableCatoTables catoTables) {
		super();
		this.catoTables = catoTables;
	}
	
	public void paint (Graphics g) {
		catoTables.paint(g);
	}
	
	public boolean mouseDown (Event e, int x, int y) {
		Enumeration arguments_enumeration = catoTables.arguments.elements();
		while (arguments_enumeration.hasMoreElements()) {
			DrawableArgument currentArgument = (DrawableArgument) arguments_enumeration.nextElement();
			if (currentArgument.within(x, y)) {
				currentArgument.select();
				catoTables.placed = false;
				repaint();
				return true;
			}
		}
		return false;
	}
}

public class DrawableCatoTables extends catoTables {

	public CatoCanvas canvas;

	public boolean placed;
	
	public static int start_at_y = 200;
	public static int gap_between_rank = 50;
	public static int gap_among_rank = 20;
	public static int left_margin = 20;
	
	private final static int maximum_depth = 100;  //problem area; I should throw an exception below if we exceed...
	private int[] rank_offset;  
	
	public DrawableArgument currentArgument;  //these are temnporary 'holding' objects
	public DrawableArrow currentArrow;  //And also a convenient way to get at statics
										//Don't confuse with "selected" record.
	private int current_id, offset;
	private Point bottom_center, starting, nextPosition;
	private Arguments arguments_to_draw;
	private Arrows arrows_to_draw;
	private Enumeration arguments_enumeration, arrows_enumeration;
	
	public DrawableCatoTables (InputStream fileStream) {
		super(fileStream);
		this.fileStream = fileStream;
		rank_offset = new int[maximum_depth];
		placed = false;
		canvas = new CatoCanvas(this);
	}
	
	/* The following methods 'place' arguments and arrows on the canvas. They work iterativly. First the
	selected argument is placed. The placement 'causes' the first arrow to be drawn, which then draws it's
	first argument, and so on, until there are no more arrows to be drawn. Then, of course, the code drops
	back down to the most 'recent' argument that has more arrows to draw. Warrants, which appear in the same
	rank (vertical position) as their related arguments, are drawn first, to avoid any problems with rank
	position. Rank offsets are incremented every time an argument is placed, and are used to determine the
	next 'available' space within a rank. This code is a little dense and repetitive; in a perfect world
	it would be less so. :-) */
	
	public void place (Graphics g) { //Initial Selected Argument placed...
		
		if (arguments.size() > 0) {
			currentArgument = (DrawableArgument) arguments.firstElement();
			if (currentArgument.selected_id == 0) {currentArgument.selected_id = currentArgument.getBaseID();}
			currentArgument = (DrawableArgument) arguments.sequentialSearch(0,currentArgument.selected_id);
			place(g, currentArgument);
		}
	}
		
	public void place (Graphics g, DrawableArgument currentArgument) {
		arguments_to_draw = (Arguments) arguments.clone();
		arrows_to_draw = (Arrows) arrows.clone();
		
		arguments_to_draw.removeElement(currentArgument);
		for (int i=0; i<rank_offset.length; i++) {
			rank_offset[i] = left_margin;
		}
		currentArgument.rank = 50;

		currentArgument.prepareForDrawing(g);
		
		starting = new Point(left_margin + (currentArgument.getExtent().x / 2), start_at_y);
		place(g, currentArgument, starting);

		arguments_to_draw.removeElement(currentArgument);
		arguments_enumeration = arguments_to_draw.elements();
		placed = true;		
	} 
	
	private void place (Graphics g, DrawableArgument argument, Point bottom_center) {
	
		arguments_to_draw.removeElement(argument);
		argument.position.y = bottom_center.y - argument.getExtent().y;
		argument.position.x = bottom_center.x - ((argument.getExtent().x)/2);
		
		rank_offset[argument.rank] = argument.position.x + argument.getExtent().x;
		argument.drawable = true;

		arrows_enumeration = arrows_to_draw.elements();
		while (arrows_enumeration.hasMoreElements()) {
	
			currentArrow = (DrawableArrow) arrows_enumeration.nextElement();
			if (currentArrow.type == currentArrow.WARRANT) {
				if (currentArrow.target_id == argument.id) {
					currentArrow.rank = argument.rank;  //Same rank
					placeFromTarget(g, currentArrow, new Point(bottom_center.x + (argument.getExtent().x / 2), bottom_center.y - (argument.getExtent().y / 2)));		
					arrows_enumeration = arrows_to_draw.elements();
				}
				if (currentArrow.origin_id == argument.id) {
					currentArrow.rank = argument.rank;
					placeFromOrigin(g, currentArrow, new Point(bottom_center.x + (argument.getExtent().x / 2), bottom_center.y - (argument.getExtent().y / 2)));
					arrows_enumeration = arrows_to_draw.elements();			
				}
			}
		}
		
		arrows_enumeration = arrows_to_draw.elements();
		while (arrows_enumeration.hasMoreElements()) {
	
			currentArrow = (DrawableArrow) arrows_enumeration.nextElement();
			if (currentArrow.target_id == argument.id) {
				currentArrow.rank = argument.rank + 1;  //An arrow shares its rank with its origin argument
				placeFromTarget(g, currentArrow, new Point(bottom_center.x, bottom_center.y - argument.getExtent().y - 1));
				arrows_enumeration = arrows_to_draw.elements();			
			}
			if (currentArrow.origin_id == argument.id) {
				currentArrow.rank = argument.rank;
				placeFromOrigin(g, currentArrow, new Point(bottom_center.x, bottom_center.y + 1));
				arrows_enumeration = arrows_to_draw.elements();
			}
		}
	}
	
	private void placeFromTarget (Graphics g, DrawableArrow arrow, Point target) {

		arrows_to_draw.removeElement(arrow);
		arguments_enumeration = arguments_to_draw.elements();
		while (arguments_enumeration.hasMoreElements()) {
	
			currentArgument = (DrawableArgument) arguments_enumeration.nextElement();
			if(currentArgument.id == arrow.origin_id) {
				currentArgument.prepareForDrawing(g);
				arrow.target = target;
				if (arrow.type != arrow.WARRANT) {
					arrow.origin.x = rank_offset[arrow.rank] + (currentArgument.getExtent().x / 2) + gap_among_rank;
					arrow.origin.y = (target.y - gap_between_rank);
					currentArgument.rank = arrow.rank + 1;
					nextPosition = new Point(arrow.origin.x, arrow.origin.y - 1);
				}
				else {
					arrow.origin.x = rank_offset[arrow.rank] + (gap_among_rank / 2);
					arrow.origin.y = target.y;					
					currentArgument.rank = arrow.rank;
					nextPosition = new Point((arrow.origin.x + currentArgument.getExtent().x / 2), (arrow.origin.y - 1) + currentArgument.getExtent().y / 2);				
				}
				arrow.drawable = true;
		
				currentArgument.rank = arrow.rank;
				place(g, currentArgument, nextPosition);
				arguments_enumeration = arguments_to_draw.elements();
			}
		}
		arrows_to_draw.removeElement(arrow);
	}
	
	private void placeFromOrigin (Graphics g, DrawableArrow arrow, Point origin) {

		arrows_to_draw.removeElement(arrow);
		arguments_enumeration = arguments_to_draw.elements();
		while (arguments_enumeration.hasMoreElements()) {
	
			currentArgument = (DrawableArgument) arguments_enumeration.nextElement();
			if(currentArgument.id == arrow.target_id) {
				currentArgument.prepareForDrawing(g);
				arrow.origin = origin;
				if (arrow.type != arrow.WARRANT) {
					arrow.target.x = rank_offset[arrow.rank - 1] + (currentArgument.getExtent().x / 2) + gap_among_rank;
					arrow.target.y = origin.y + gap_between_rank;
					nextPosition = new Point(arrow.target.x, arrow.target.y + currentArgument.getExtent().y + 1);						
					currentArgument.rank = arrow.rank - 1;
				}
				else {
					arrow.target.x = rank_offset[arrow.rank] + gap_among_rank + (gap_among_rank / 2);
					arrow.target.y = arrow.origin.y;					
					nextPosition = new Point((arrow.target.x + currentArgument.getExtent().x / 2), (arrow.target.y - 1) + currentArgument.getExtent().y / 2);
					currentArgument.rank = arrow.rank;
				}
				arrow.drawable = true;
		
				place(g, currentArgument, nextPosition);
				arguments_enumeration = arguments_to_draw.elements();
			}
		}
		arrows_to_draw.removeElement(arrow);
	}
	
	public void addArgument (Graphics g, int type) {
		Frame f = new Frame("Request Dialog Test");
		f.resize(200, 300);
		RequestDialog d = new RequestDialog(f, g, this, type); //invokes the method below;
		d.show();
	}
	
	public void addArgument (Graphics g, String text, int type) { //assumes a current selection

		int target_id = 0;
		
		if (arguments.size() > 0) {
			currentArgument = (DrawableArgument) arguments.firstElement();
			if (currentArgument.selected_id == 0) {currentArgument.selected_id = currentArgument.getBaseID();}
			currentArgument = (DrawableArgument) arguments.sequentialSearch(0,currentArgument.selected_id);
			if(currentArgument != null) {
				target_id = currentArgument.id;
			}
		}
		
		currentArgument = new DrawableArgument(text);
		catoTables.arguments.addElement(currentArgument);
		
		if (target_id != 0) {  //A target has been defined
			currentArrow = new DrawableArrow(target_id, currentArgument.id, type);
			catoTables.arrows.addElement(currentArrow);
		}
		
		place (g, currentArgument);
	}
	
	public void paint (Graphics g) {
	
		if (!placed) place(g);
		for(int i=0; i<tables.length; i++) {	
			Enumeration enumeration = tables[i].elements();
			while (enumeration.hasMoreElements()) {
				Drawable record = (Drawable) enumeration.nextElement();
				record.paint(g);
			}
		}
	}
}
