import java.util.*;
import java.io.*;
import java.lang.Math;
import java.net.*;

import Database;
import DSRow;

public class Dataset {
  public Database db;
  public String query;
  public int rowSize;
  public int rowCount;
  public int colCount;
  public int rowCurrent;
  public byte rowState[];
  public byte colType[];
  public String colName[];
  public int colOffset[];
  public int colSize[];
  public int colLen[];
  public byte rowData[];
  public DSRow row;
  public DataInputStream qb;
  public int blockSize;

  static int atoi(String s) {
    return (new Integer(s)).intValue();
  }


  Dataset() {
	db = null;
	query = null;
	colCount = 0;
	rowCount = 0;
  }

  Dataset(Database dbname, String sql) {
    db = dbname;
    query = sql;
    if (go()) {
      System.err.println("Query ("+sql+") didn't work");
    }
    gotoRow(0);
  }

  public String toString() {
    byte i;
    byte j;
    StringBuffer out = new StringBuffer(rowSize*rowCount*5);

    for (i=0; i < rowCount; i++) {
      for (j=0; j < rowSize; j++) {
        out.append(rowData[i*rowSize+j]).append(", ");
      }
      out.append('\n');
    }
    return out.toString();
  }

  boolean getQueryFields() {
    rowSize = 0;

    try {
	    byte b1 = db.din.readByte();
	    byte b2 = db.din.readByte();
	    colCount = (short)(b1+b2*256);

	    if (colCount <= 0) {
	      return(true);
	    }

	    colType = new byte[colCount];
	    colSize = new int[colCount];
	    colName = new String[colCount];
	    colOffset = new int[colCount];

	    for (int i=0; i < colCount; i++) {
	      colOffset[i] = rowSize;
	      colType[i] = db.din.readByte();
    	  b1 = db.din.readByte();
	      b2 = db.din.readByte();
	      colSize[i] = (short)(b1+b2*256);

	      if (colType[i] == DSRow.REAL_TYPE) {colSize[i]=4;}
 	      if (colType[i] == DSRow.CHAR_TYPE) {colSize[i]+=2;}

	      colName[i] = new String(db.din.readUTF());
	      rowSize += (short) colSize[i];

	      if (colType[i] == DSRow.INT_TYPE) {rowSize += 2;}
	    }

    } catch (IOException e) {
    	e.printStackTrace();
      	System.err.println(e.getMessage());
    }

    return(false);
  }

  boolean getRows() {
  	try {
    	int b1 = (int)db.din.readByte();
    	if (b1 <0) {b1 = b1+256;} // Compensate for signed bytes
	    int b2 = (int)db.din.readByte();
    	if (b2 <0) {b2 = b2+256;} // Compensate for signed bytes
    	rowCount = (int)(b1+b2*256);
    	blockSize = rowCount * rowSize;
    	rowData = new byte[blockSize];
	    db.din.readFully(rowData, 0, blockSize);
    	db.din.readByte();
	    qb = new DataInputStream(new ByteArrayInputStream(rowData));
    } catch (IOException e) {
    	e.printStackTrace();
      	System.err.println(e.getMessage());
    }

    return(false);
  }

  public boolean go() {
    byte stat;
    try {
    	db.dout.writeByte(db.MSG_QUERY);
    	db.dout.writeUTF(query);
        	db.dout.flush();
    	if (getQueryFields()) {
        	  return(true);
    	}
    	if (getRows()) {
          		return(true);
        }
    	row = new DSRow(qb, colCount, colType, colSize);
    	rowCurrent = 0;
    } catch (IOException e) {
    	if (e.getClass().equals("EOFException")) {
 	       	return(true);
    	} else {
	    	e.printStackTrace();
    	  	System.err.println(e.getMessage());
    	}
    }

    return(false);
  }

  public boolean gotoRow(int rowNum) {
    if (rowNum >= rowCount) {
       return(true);
    }
    try {
	    qb.reset();
	    qb.skip(rowNum*rowSize);
	    row.fetch();
    } catch (IOException e) {
    	e.printStackTrace();
      	System.err.println(e.getMessage());
    }
    return(false);
  }
}
