import java.awt.*;
import java.util.Vector;

class DBList extends List implements QueryableComponent {

    DBPanel     panel;
    Database    db;
    String      sql;
    Dataset     ds;
    boolean     useAlias;
    Vector      valAlias;

    DBList(int visRows, Database rdbms, String query) {
	    super(visRows, false);
	    valAlias = new Vector();
	    setSQLText(rdbms, query);
	    ds = new Dataset(db, sql);
	    useAlias = (ds.colCount > 1);
        populate();
    }

    DBList(int visRows) {
	    super(visRows, false);
	    valAlias = new Vector();
	    useAlias = false;
	    setSQLText(null, null);
	    ds = null;
    }

    // Implement QueryableComponent Interface

    public void setDBPanel(DBPanel dbPanel) {
        panel = dbPanel;
    }

    public DBPanel getDBPanel() {
        return panel;
    }

    public void hasChangedEvent() {
        panel.handleChange();
    }

    public byte getWidgetType() {
        return LIST_TYPE;
    }

    public String getSQLText() {
        return sql;
    }

    public void setSQLText(Database rdbms, String query) {
        sql = query;
        db = rdbms;
    }

    public void refreshSQL() {
        reset();
	    ds = new Dataset(db, sql);
	    useAlias = (ds.colCount > 1);
        populate();
    }

    public void refreshSQL(Database rdbms, String query) {
        reset();
	    ds = new Dataset(rdbms, query);
	    useAlias = (ds.colCount > 1);
        populate();
    }

    public Dataset getDataset() {
        return ds;
    }

    public int getRownum() {
        return getSelectedIndex();
    }

    public String getValue() {
        if ((getSelectedItem() == null) && (countItems() > 0)) {
            select(0);
        }
        return (String)valAlias.elementAt(getSelectedIndex());
    }

    public String getValueAlias() {
        if ((getSelectedItem() == null) && (countItems() > 0)) {
            select(0);
        }
        return getSelectedItem();
    }

    public void reset() {
        delItems(0, countItems()-1);
        if (valAlias != null) {
            valAlias.removeAllElements();
        }
    }

    // Add each row in Dataset to List object

    private void populate() {

    	for (int r=0; r < ds.rowCount; r++) {
    		ds.gotoRow(r);
    		if (useAlias) {
    		    addItem(ds.row.col[1].toString()); // Display Alias in GUI
    		    valAlias.addElement(ds.row.col[0].toString());
    		} else {
    		    addItem(ds.row.col[0].toString());
    		    valAlias.addElement(ds.row.col[0].toString());
            }

    	}
    }

    /*
     * Add sql output to the parameter String of this list.
     */

    protected String paramString() {
	    return super.paramString() + ",sql=" + getSQLText();
	}


}