import java.awt.*;

class DBTextField extends TextField implements QueryableComponent {

    DBPanel     panel;
    Database    db;
    String      sql;
    Dataset     ds;

    DBTextField(Database rdbms, String query) {

	    super();
	    setSQLText(rdbms, query);
	    ds = new Dataset(db, sql);
        populate();
    }

    DBTextField() {

	    super();
	    setSQLText(null, null);
	    ds = null;
    }

   // Implement QueryableComponent Interface

    public void setDBPanel(DBPanel dbPanel) {
        panel = dbPanel;
    }

    public DBPanel getDBPanel() {
        return panel;
    }

    public void hasChangedEvent() {
        panel.handleChange();
    }

    public byte getWidgetType() {
        return TEXT_FIELD_TYPE;
    }

    public String getSQLText() {
        return sql;
    }

    public void setSQLText(Database rdbms, String query) {
        sql = query;
        db = rdbms;
    }

    public void refreshSQL() {
	    ds = new Dataset(db, sql);
        populate();
    }

    public void refreshSQL(Database rdbms, String query) {
	    ds = new Dataset(db, sql);
        populate();
    }

     public Dataset getDataset() {
        return ds;
    }

    public int getRownum() {
        return 0;
    }

    public String getValue() {
        return getText();
    }

    // Add each row in Dataset to Choice object

    private void populate() {
        // Set default value of TextField to first column
        // of the first row in the Dataset

        if (ds.rowCount > 0) {
   		    ds.gotoRow(0);
    	    setText(ds.row.col[0].toString());
    	}
    }

    /*
     * Add sql output to the parameter String of this list.
     */

    protected String paramString() {
	    return super.paramString() + ",sql=" + getSQLText();
	}

}